/* Copyright (C) 2010-2014 Frankfurt Institute for Advanced Studies, Goethe-Universität Frankfurt, Frankfurt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Igor Kulakov [committer], Maksym Zyzak */

#pragma once  // include this header only once per compilation unit

#include <boost/serialization/access.hpp>
#include <boost/serialization/array.hpp>
#include <boost/serialization/split_free.hpp>

#include <Vc/Vc>

namespace cbm::algo::ca
{
  typedef Vc::float_v fvec;
  //typedef Vc::double_v fvec;
  //typedef Vc::Vector<float, Vc::VectorAbi::Scalar>  fvec;
  //typedef Vc::SimdArray<float, 4> fvec;

  typedef fvec::EntryType fscal;
  typedef fvec::MaskType fmask;

#define _fvecalignment __attribute__((aligned(Vc::VectorAlignment)))
}  // namespace cbm::algo::ca

/// \brief Serializer for SIMD vectors
namespace boost::serialization
{
  template<class Archive>
  void save(Archive& ar, const cbm::algo::ca::fvec& vect, unsigned int)
  {
    std::array<cbm::algo::ca::fscal, cbm::algo::ca::fvec::size()> buffer;
    for (size_t i = 0; i < cbm::algo::ca::fvec::size(); ++i) {
      buffer[i] = vect[i];
    }
    ar << buffer;
  }

  template<class Archive>
  void load(Archive& ar, cbm::algo::ca::fvec& vect, unsigned int)
  {
    std::array<cbm::algo::ca::fscal, cbm::algo::ca::fvec::size()> buffer;
    ar >> buffer;
    for (size_t i = 0; i < cbm::algo::ca::fvec::size(); ++i) {
      vect[i] = buffer[i];
    }
  }

  template<class Archive>
  void serialize(Archive& ar, cbm::algo::ca::fvec& vect, const unsigned int version)
  {
    split_free(ar, vect, version);
  }
}  // namespace boost::serialization
